@@ -4,13 +4,10 @@ import copy
 import enum
 import logging
 from typing import Any
-from typing import Dict
-from typing import List
 from typing import Optional
 from urllib import parse
 
 from certbot import errors
-from certbot import util
 from certbot._internal import constants
 from certbot.compat import misc
 from certbot.compat import os
@@ -73,7 +70,7 @@ class NamespaceConfig:
         # Check command line parameters sanity, and error out in case of problem.
         _check_config_sanity(self)
 
-    def set_argument_sources(self, argument_sources: Dict[str, ArgumentSource]) -> None:
+    def set_argument_sources(self, argument_sources: dict[str, ArgumentSource]) -> None:
         """
         Associate the NamespaceConfig with a dictionary describing where each of
         its arguments came from, e.g. `{ 'email': ArgumentSource.CONFIG_FILE }`.
@@ -133,7 +130,7 @@ class NamespaceConfig:
 
         return False
 
-    def to_dict(self) -> Dict[str, Any]:
+    def to_dict(self) -> dict[str, Any]:
         """
         Returns a dictionary mapping all argument names to their values
         """
@@ -152,7 +149,7 @@ class NamespaceConfig:
                 del self._previously_accessed_mutables[name]
 
     @property
-    def argument_sources(self) -> Optional[Dict[str, ArgumentSource]]:
+    def argument_sources(self) -> Optional[dict[str, ArgumentSource]]:
         """Returns _argument_sources after handling any changes to accessed mutable values."""
         # We keep values in _previously_accessed_mutables until we've detected a modification to try
         # to provide up-to-date information when argument_sources is accessed. Once a mutable object
@@ -297,7 +294,7 @@ class NamespaceConfig:
     def no_verify_ssl(self) -> bool:
         """Disable verification of the ACME server's certificate.
 
-        The root certificates trusted by Certbot can be overriden by setting the
+        The root certificates trusted by Certbot can be overridden by setting the
         REQUESTS_CA_BUNDLE environment variable.
         """
         return self.namespace.no_verify_ssl
@@ -326,7 +323,7 @@ class NamespaceConfig:
         return self.namespace.https_port
 
     @property
-    def pref_challs(self) -> List[str]:
+    def pref_challs(self) -> list[str]:
         """List of user specified preferred challenges.
 
         Sorted with the most preferred challenge listed first.
@@ -485,12 +482,6 @@ def _check_config_sanity(config: NamespaceConfig) -> None:
             "Trying to run http-01 and https-port "
             "on the same port ({0})".format(config.https_port))
 
-    # Domain checks
-    if config.namespace.domains is not None:
-        for domain in config.namespace.domains:
-            # This may be redundant, but let's be paranoid
-            util.enforce_domain_sanity(domain)
-
 
 def _is_immutable(value: Any) -> bool:
     """Is value of an immutable type?"""
